#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-03-31
#
# Monitors Cisco WLC memory utilization
#
# tested with WLC2504
#
# 2016-03-31: initial release
# 2016-07-02: fixed crit/warn to >=
# 2021-07-15: rewritten for CMK 2.0

from dataclasses import dataclass
from typing import Optional
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    SNMPTree,
    contains,
    State,
    render,
    check_levels,
)


@dataclass
class CiscoWlcMem:
    totalmemory: int
    freememory: int
    usedmemory: int
    percentusedmemory: float


def parse_cisco_wlc_mem(string_table: StringTable) -> Optional[CiscoWlcMem]:
    try:
        totalmemory, freememory = string_table[0]
    except IndexError:
        return

    totalmemory = int(totalmemory) * 1024  # chenage from Kbytes to Bytes
    freememory = int(freememory) * 1024  # chenage from Kbytes to Bytes
    usedmemory = totalmemory - freememory
    percentusedmemory = 100 * usedmemory / totalmemory

    return CiscoWlcMem(
        totalmemory=totalmemory,
        freememory=freememory,
        usedmemory=usedmemory,
        percentusedmemory=percentusedmemory
    )


def discovery_cisco_wlc_mem(section: CiscoWlcMem) -> DiscoveryResult:
    yield Service()


def check_cisco_wlc_mem(params, section: CiscoWlcMem) -> CheckResult:
    yield from check_levels(
        value=section.percentusedmemory,
        label='Memory utilization',
        levels_upper=params['levels_upper'],
        render_func=render.percent,
        metric_name='cisco_wlc_mem_utilization',
        boundaries=(0, 100),
    )

    usedmemory_mb = section.usedmemory / 1024 / 1024
    totalmemory_mb = section.totalmemory / 1024 / 1024
    freememory_mb = section.freememory /1024 / 1024

    yield Result(state=State.OK, notice=f'(Memory total: {totalmemory_mb:.2f}MB, '
                                        f'used: {usedmemory_mb:.2f}MB, '
                                        f'free: {freememory_mb:.2f}MB)')


register.snmp_section(
    name='cisco_wlc_mem',
    parse_function=parse_cisco_wlc_mem,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.14179.1.1.5',  # AIRESPACE-WIRELESS-MIB
        oids=[
            '2',  # agentTotalMemory in Kbytes
            '3',  # agentFreeMemory in Kbytes
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'Cisco Controller'),  # sysDescr
)

register.check_plugin(
    name='cisco_wlc_mem',
    service_name='Memmory utilization',
    discovery_function=discovery_cisco_wlc_mem,
    check_function=check_cisco_wlc_mem,
    check_default_parameters={'levels_upper': (50, 70)},
    check_ruleset_name='cisco_wlc_mem',
)
